/**
 *	@file		PololuQik2.cpp
 *	@author		Edward Wilson (edwilson1989@gmail.com)
 *  @date 		Aug 6, 2010
 *  @brief		 
 */

/*************************************************************
 PololuQik2 - basic class to control Pololu's Qik2s9v1
 motor controller (http://www.pololu.com/catalog/product/1110)

 This uses the default settings for the motor controller and the
 Compact Protocol to communicate to it.

 This library also depends on the NewSoftSerial library which
 can be found at: http://arduiniana.org.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 Author:  Edward Wilson (edwilson1989@gmail.com)
 History:
 Version 1.0	August 6, 2010		Initial code

************************************************************/

#include "PololuQik2.h"

/
void PololuQik2::motor1Reverse(uint8_t speed) {
	unsigned char message[] = { MOTOR1REVERSEPACKET, speed };
	sendMessage(message, 2);
}

void PololuQik2::motor1Forward(uint8_t speed) {
	unsigned char message[] = { MOTOR1FORWARDPACKET, speed };
	sendMessage(message, 2);
}

void PololuQik2::motor0Coast() {
	unsigned char message[] = { MOTOR0COASTPACKET };
	sendMessage(message, 1);
}

void PololuQik2::motor0Forward(uint8_t speed) {
	unsigned char message[] = { MOTOR0FORWARDPACKET, speed };
	sendMessage(message, 2);
}

void PololuQik2::begin() {
	digitalWrite(_resetPin, LOW);
	delay(100);
	digitalWrite(_resetPin, HIGH);
	delay(10);

	unsigned char message[] = { INITIALPACKET };
	sendMessage(message, 1);

	delay(100);
}

void PololuQik2::sendMessage(unsigned char message[], unsigned int length) {
	if (_enCRC) {
		uint8_t chk = _crc->CRC(message, length);
		for (int i = 0; i < length; i++)
			sendByte(message[i]);

		sendByte(chk);
	} else {
		for (int i = 0; i < length; i++)
			sendByte(message[i]);
	}
}

void PololuQik2::stopMotor0() {
	unsigned char message[] = { MOTOR1REVERSEPACKET, 0 };
	sendMessage(message, 2);
}

void PololuQik2::stopBothMotors() {
	stopMotor0();
	stopMotor1();
}

unsigned int PololuQik2::getFirmwareVersion() {
	unsigned char message[] = { FWVERSIONPACKET };
	sendMessage(message, 1);

	return readByte();
}

PololuQik2::PololuQik2(NewSoftSerial *serial, uint8_t resetPin, uint8_t errPin, bool enCRC, CRC7 *crc) {
	_serial = serial;
	_resetPin = resetPin;
	_errByte = -1;
	_errPin = errPin;
	_enCRC = enCRC;
	_crc = crc;

	_crc.init();
	pinMode(_resetPin, OUTPUT);

	// Interrupt section will go here
	if(errPin == 2 || errPin ==3){
		 //attachInterrupt(errPin -2, PololuQik2::error, RISING);
	}

}

void PololuQik2::motor1Coast() {
	unsigned char message[] = { MOTOR1COASTPACKET };
	sendMessage(message, 1);
}

void PololuQik2::stopMotor1() {
	unsigned char message[] = { MOTOR1REVERSEPACKET, 0 };
	sendMessage(message, 2);
}

void PololuQik2::motor0Reverse(uint8_t speed) {
	unsigned char message[] = { MOTOR0REVERSEPACKET, speed };
	sendMessage(message, 2);
}

bool PololuQik2::hasCRCError() {
	getError();
	return errorBitSet(CRCERRORBIT);
}

bool PololuQik2::errorBitSet(uint8_t bitToCheck) {
	getError();
	return (1 << bitToCheck) & _errByte; //bit shift
}

bool PololuQik2::hasFrameError() {
	getError();
	return errorBitSet(FRAMEERRORBIT);
}

bool PololuQik2::hasDataOverrunError() {
	getError();
	return errorBitSet(DATAOVERRUNERRORBIT);
}

bool PololuQik2::hasFormatError() {
	getError();
	return errorBitSet(FORMATERRORBIT);
}

bool PololuQik2::hasTimeoutError() {
	getError();
	return errorBitSet(TIMEOUTERRORBIT);
}

uint8_t PololuQik2::getError() {
	unsigned char message[] = { ERRORPACKET };
	sendMessage(message, 1);
	_errByte = readByte();
	return _errByte;
}

void PololuQik2::error(){
	Serial.print("An Error has Occurred: ");
	Serial.println(getError());
}

uint8_t PololuQik2::readByte(){
	return _serial->read();
}

void PololuQik2::sendByte(uint8_t byte){
	_serial->print(byte, BYTE);
}
