/**
 *	@file		PololuQik2.h
 *	@author		Edward Wilson (edwilson1989@gmail.com)
 *  @date 		Aug 6, 2010
 *  @brief		 
 */

/**
 * @mainpage
 * PololuQik2 - basic class to control Pololu's Qik2s9v1
 * motor controller (http://www.pololu.com/catalog/product/1110)
 *
 * This uses the default settings for the motor controller and the
 * Compact Protocol to communicate to it.
 *
 * This library also depends on the NewSoftSerial library which
 * can be found at: http://arduiniana.org.
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Author:  Edward Wilson (edwilson1989@gmail.com)
 *
 * Many thanks to Eric Fialkowski (eric.fialkowski@gmail.com) whose work this is based on.

 * @Version 0.1
 * @date August 6, 2010
 * @attention To use this you will need NewSoftSerial Library and CRC7 library installed.
 *
 * @brief This class allows an arduino to communicate with and control a pololu Qik2 motor
 * controller module.
 */


#ifndef POLOLUQIK2_H_
#define POLOLUQIK2_H_

#include "../NewSoftSerial/NewSoftSerial.h"
#include "../CRC7/CRC7.h"
#include "WProgram.h"
#include <inttypes.h>
#include <avr/interrupt.h>

#define INITIALPACKET 0xAA
#define MOTOR0FORWARDPACKET 0x88
#define MOTOR1FORWARDPACKET 0x8C
#define MOTOR0REVERSEPACKET 0x8A
#define MOTOR1REVERSEPACKET 0x8E
#define MOTOR0COASTPACKET 0x86
#define MOTOR1COASTPACKET 0x87
#define FWVERSIONPACKET 0x81
#define ERRORPACKET 0x82

#define DATAOVERRUNERRORBIT 3
#define FRAMEERRORBIT 4
#define CRCERRORBIT 5
#define FORMATERRORBIT 6
#define TIMEOUTERRORBIT 7

/**
 * This class provides the ability for an arduino to communicate with a pololu Qik2s9v1.
 * It offers both CRC and non-CRC operating modes. Interrupts is partly written in. To use
 * this you will need NewSoftSerial Library and CRC7 library installed.
 */
class PololuQik2 {
public:

	/**
	 * Constructor for the class. Requires that a NewSoftSerial and CRC7 Object are passed to it.
	 *
	 *	This constructor also calls
	 * @param *serial a NewSoftSerial object provides communication to the module.
	 * @param reset is the pin the reset on the module is connected to on the arduino.
	 * @param errPin must be an interrupt pin. This feature is not fully implemented.
	 * @param EnableCRC is a boolean value as to wheather or not CRC is enabled.
	 * @param *crc a CRC7 object which can be used for hashing the data. If this is not reqired set to NULL
	 *
	 * @todo Implement errPin fully.
	 */
	PololuQik2(NewSoftSerial *serial, uint8_t reset, uint8_t errPin, bool EnableCRC, CRC7 *crc);

	/**
	 * Must be called before the pololu module is used. This resets the module and sets up communication
	 * with it.
	 *
	 * It is recommended that this command is called in the setup() method.
	 */
	void begin();

	/**
	 * Makes Motor 0 go forwards with a speed of speed.
	 *
	 * @param speed is the speed you wish the motor to move at. Acceptable values are between 0 and 127.
	 */
	void motor0Forward(uint8_t speed);

	/**
	 * Makes Motor 1 go forwards with a speed of speed.
	 *
	 * @param speed is the speed you wish the motor to move at. Acceptable values are between 0 and 127.
	 */
	void motor1Forward(uint8_t speed);

	/**
	 * Makes Motor 0 reverse with a speed of speed.
	 *
	 * @param speed is the speed you wish the motor to move at. Acceptable values are between 0 and 127.
	 */
	void motor0Reverse(uint8_t speed);

	/**
	 * Makes Motor 1 reverse with a speed of speed.
	 *
	 * @param speed is the speed you wish the motor to move at. Acceptable values are between 0 and 127.
	 */
	void motor1Reverse(uint8_t speed);

	/**
	 * Stops both motors. This method actually calls stopMotor0 and stopMotor1
	 *
	 * @see stopMotor0
	 * @see stopMotor1
	 *
	 */
	void stopBothMotors();

	/**
	 * Stops Motor 0. This actually called motor0Forward with a speed value of 0.
	 *
	 * @see motor0Forward
	 */
	void stopMotor0();

	/**
	 * Stops Motor 1. This acutually called motor1Forward with a speed value of 0.
	 *
	 * @see motor0Forward
	 */
	void stopMotor1();

	/**
	 * Allows motor 0 to coast freely.
	 */
	void motor0Coast();

	/**
	 * Allows motor 1 to coast freely.
	 */
	void motor1Coast();

	/**
	 * returns the firmware version
	 *
	 * @return unsigned int firmware version
	 */
	unsigned int getFirmwareVersion();

	/**
	 * Checks if there is a data overrun error.
	 *
	 * @return boolean - true if the error has occurred.
	 */
	bool hasDataOverrunError();

	/**
	 * Checks if there is a Frame Error.
	 *
	 * @return boolean - true if the error has occurred.
	 */
	bool hasFrameError();

	/**
	 * Checks if there is a CRC error occurred.
	 *
	 * @return boolean - true if the error has occurred.
	 */
	bool hasCRCError();

	/**
	 * Checks if there is a Format Error has occurred.
	 *
	 * @return boolean - true if the error has occurred.
	 */
	bool hasFormatError();

	/**
	 * Checks if there is a timeout error has occurred.
	 *
	 * @return boolean - true if the error has occurred.
	 */
	bool hasTimeoutError();

private:

	/**
	 * formats and sends the packets of data to the controller
	 *
	 * @param message[] an array of bytes to be sent.
	 * @param length the size of the array.
	 */
	void sendMessage(unsigned char message[], unsigned int length);

	/**
	 * checks if the bitToCheck bit is set. This is used to determine which type of
	 * error has occurred.
	 *
	 * @param bitToCheck is a uint8_t. The bit will be tested
	 * @return true if the bit is set.
	 */
	bool errorBitSet(uint8_t bitToCheck);

	/**
	 * Gets and stores the error from the error register on the module.
	 * The error is stored in _errByte.
	 * @return uint8_t which details the error that has occurred.
	 * @see _errByte
	 */
	uint8_t getError();

	/**
	 * is the part implementation of interrupts. Ideally this method will be called when an error occurrs.
	 * This should happen automatically. This should alert the system/user to the error. The arduino
	 * support interrupts however a persistence error prevents this from working.
	 *
	 * @note You may be able to get this working. My C++ is functional (excuse the pun) but I am no
	 * way an expert.
	 */
	void error();

	/**
	 * sends a byte out of the NewSoftSerial interface.
	 *
	 * @param byte is the byte to be sent
	 */
	void sendByte(uint8_t byte);

	/**
	 * Reads a byte from the NewSoftSerial interface. This is a blocking call. If this is called and no
	 * data is being sent then it will halt the execution of the program until it recieves a byte.
	 *
	 * @return a byte from the software serial interface
	 */
	uint8_t readByte();

	/**
	 * This is an instance of NewSoftSerial which the arduino will expect to find a pololu Qik2 connected
	 * to.
	 */
	NewSoftSerial *_serial;

	/**
	 * is the arduino pin which is connected to the reset pin on the pololu qik2 module.
	 */
	uint8_t _resetPin;

	/**
	 * Stores the error byte once an error has occurred.
	 */
	uint8_t _errByte;

	/**
	 * stores the firmware version. This is populated with the value onces getFirmwareVersion is called.
	 * After that it will simple recall the value from memory. It suppose to save space it could query the
	 * module each time. This would increase serial traffic and possibly slow the system down. Its a compromise.
	 *
	 */
	uint8_t _fwVersion;

	/**
	 * is an interrupt pin on the arduino which is connected to the error pin on the module.
	 * This is not currently implemented correctly.
	 *
	 * @attention This is not fully functional
	 * @todo Automated Error Detection needs to be completed.
	 */
	uint8_t _errPin;

	/**
	 * is true if CRC is enabled.
	 */
	bool _enCRC;

	/**
	 * a pointer to a CRC7 object. This is used to hash the data being sent to the module. CRC can be used
	 * for error detection. And in advanced cases error correction.
	 */
	CRC7 *_crc;


};

#endif /* POLOLUQIK2_H_ */
